/*

*/

import renderer.scene.*;
import renderer.pipeline.*;
import renderer.framebuffer.*;

import java.awt.Color;

/**
<pre>{@code
                  y
                  |
                  |
                  | v[4]=(0,1,0)
                1 +---------------------+ v[5]=(1,1,0)
                 /|                    /|
                / |                   / |
               /  |                  /  |
              /   |                 /   |
             /    |                /    |
       v[7] +---------------------+ v[6]|
            |     |               |     |
            |     |               |     |
            |     | v[0]=(0,0,0)  |     | v[1]=(1,0,0)
            |     +---------------|-----+------> x
            |    /                |    /1
            |   /                 |   /
            |  /                  |  /
            | /                   | /
            |/                    |/
          1 +---------------------+
           /v[3]=(0,0,1)          v[2]=(1,0,1)
          /
         /
        z
}</pre>
*/
public class OfflineCube
{
   public static void main(String[] args)
   {
      // Create the Scene object that we shall render
      final Scene scene = new Scene();

      // Create a Model object to hold the geometry.
      final Model model = new Model("cube");

      // Add the Model to the Scene.
      scene.addModel(model);

      // Create the geometry for the Model.
      final Vertex v0 = new Vertex(0.0, 0.0, 0.0); // four vertices around the bottom face
      final Vertex v1 = new Vertex(1.0, 0.0, 0.0);
      final Vertex v2 = new Vertex(1.0, 0.0, 1.0);
      final Vertex v3 = new Vertex(0.0, 0.0, 1.0);
      final Vertex v4 = new Vertex(0.0, 1.0, 0.0); // four vertices around the top face
      final Vertex v5 = new Vertex(1.0, 1.0, 0.0);
      final Vertex v6 = new Vertex(1.0, 1.0, 1.0);
      final Vertex v7 = new Vertex(0.0, 1.0, 1.0);

      model.addVertex(v0, v1, v2, v3, v4, v5, v6, v7);

      // Add the geometry to the Model.
      model.addLineSegment(new LineSegment(0, 1),  // bottom face
                           new LineSegment(1, 2),  // bottom face
                           new LineSegment(2, 3),  // bottom face
                           new LineSegment(3, 0),  // bottom face
                           new LineSegment(4, 5),  // top face
                           new LineSegment(5, 6),  // top face
                           new LineSegment(6, 7),  // top face
                           new LineSegment(7, 4),  // top face
                           new LineSegment(0, 4),  // back face
                           new LineSegment(1, 5),  // back face
                           new LineSegment(3, 7),  // front face
                           new LineSegment(2, 6)); // front face

      // Push the model away from where the camera is.
      for (int i = 0; i < model.vertexList.size(); ++i)
      {
         final Vertex v = model.vertexList.get(i);
         model.vertexList.set(i, new Vertex(v.x,
                                            v.y,
                                            v.z - 2));
      }

      // Log interesting information to standard output.
      //System.out.println( scene.toString() );

      // Create a FrameBuffer to render our scene into.
      final int width  = 1024;
      final int height = 1024;
      final FrameBuffer fb = new FrameBuffer(width, height);

      // Render our scene into the frame buffer once with debugging.
      fb.clearFB(Color.darkGray);
      Pipeline.debug  = true;
      Pipeline.render(scene, fb);
      fb.dumpFB2File(String.format("PPM_OfflineCube_Frame%03d.ppm", 0));
      Pipeline.debug  = false;
      Rasterize_Clip.doClipping = false; // turn off clipping

      for (int j = 1; j <= 100; ++j)
      {
         // Update the scene by translating in
         // camera space each of the 8 vertices.
         for (int i = 0; i < model.vertexList.size(); ++i)
         {
            final Vertex v = model.vertexList.get(i);
            model.vertexList.set(i, new Vertex(v.x + 0.1,
                                               v.y,
                                               v.z));
         }

         // Render into the framebuffer again.
         fb.clearFB(Color.darkGray); // What happens if we don't clear the framebuffer?
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_OfflineCube_Frame%03d.ppm", j));
      }
   }
}
