/*

*/

import renderer.scene.*;
import renderer.pipeline.*;
import renderer.framebuffer.*;

import java.awt.Color;

/**
<pre>{@code
                  y
                  |
                  |
                  | v[4]
                1 +---------------------+ v[5]=(1,1,0)
                 /|                    /|
                / |                   / |
               /  |                  /  |
              /   |                 /   |
             /    |                /    |
       v[7] +---------------------+ v[6]|
            |     |               |     |
            |     |               |     |
            |     | v[0]          |     | v[1]
            |     +---------------|-----+------> x
            |    /                |    /1
            |   /                 |   /
            |  /                  |  /
            | /                   | /
            |/                    |/
          1 +---------------------+
           /v[3]=(0,0,1)          v[2]=(1,0,1)
          /
         /
        z
}</pre>
*/
public class OfflineCubes
{
   public static void main(String[] args)
   {
      // Create the Scene object that we shall render
      final Scene scene = new Scene();

      // Create Model objects to hold the geometry.
      final Model model1 = new Model();
      final Model model2 = new Model();

      // Add the Models to the Scene.
      scene.addModel(model1, model2);

      // Add geometry to the Models
      // First cube
      final Vertex v0 = new Vertex(0.0, 0.0, 0.0);  // four vertices around the bottom face
      final Vertex v1 = new Vertex(1.0, 0.0, 0.0);
      final Vertex v2 = new Vertex(1.0, 0.0, 1.0);
      final Vertex v3 = new Vertex(0.0, 0.0, 1.0);
      final Vertex v4 = new Vertex(0.0, 1.0, 0.0);  // four vertices around the top face
      final Vertex v5 = new Vertex(1.0, 1.0, 0.0);
      final Vertex v6 = new Vertex(1.0, 1.0, 1.0);
      final Vertex v7 = new Vertex(0.0, 1.0, 1.0);

      // Second cube.
      final Vertex w0 = new Vertex(0.0, 0.0, 0.0);  // four vertices around the bottom face
      final Vertex w1 = new Vertex(2.0, 0.0, 0.0);
      final Vertex w2 = new Vertex(2.0, 0.0, 2.0);
      final Vertex w3 = new Vertex(0.0, 0.0, 2.0);
      final Vertex w4 = new Vertex(0.0, 2.0, 0.0);  // four vertices around the top face
      final Vertex w5 = new Vertex(2.0, 2.0, 0.0);
      final Vertex w6 = new Vertex(2.0, 2.0, 2.0);
      final Vertex w7 = new Vertex(0.0, 2.0, 2.0);

      model1.addVertex(v0, v1, v2, v3, v4, v5, v6, v7);
      model2.addVertex(w0, w1, w2, w3, w4, w5, w6, w7);

      // First cube.
      model1.addLineSegment(new LineSegment(0, 1),  // bottom face
                            new LineSegment(1, 2),  // bottom face
                            new LineSegment(2, 3),  // bottom face
                            new LineSegment(3, 0),  // bottom face
                            new LineSegment(4, 5),  // top face
                            new LineSegment(5, 6),  // top face
                            new LineSegment(6, 7),  // top face
                            new LineSegment(7, 4),  // top face
                            new LineSegment(0, 4),  // back face
                            new LineSegment(1, 5),  // back face
                            new LineSegment(3, 7),  // front face
                            new LineSegment(2, 6)); // front face

      // Second cube.
      model2.addLineSegment(new LineSegment(0, 1),  // bottom face
                            new LineSegment(1, 2),  // bottom face
                            new LineSegment(2, 3),  // bottom face
                            new LineSegment(3, 0),  // bottom face
                            new LineSegment(4, 5),  // top face
                            new LineSegment(5, 6),  // top face
                            new LineSegment(6, 7),  // top face
                            new LineSegment(7, 4),  // top face
                            new LineSegment(0, 4),  // back face
                            new LineSegment(1, 5),  // back face
                            new LineSegment(3, 7),  // front face
                            new LineSegment(2, 6)); // front face

      // Push the models away from where the camera is.
      for (int i = 0; i < model1.vertexList.size(); ++i)
      {
         final Vertex v = model1.vertexList.get(i);
         model1.vertexList.set(i, new Vertex(v.x,
                                             v.y,
                                             v.z - 4));
      }
      for (int i = 0; i < model2.vertexList.size(); ++i)
      {
         final Vertex v = model2.vertexList.get(i);
         model2.vertexList.set(i, new Vertex(v.x,
                                             v.y,
                                             v.z - 4));
      }

      // Log interesting information to standard output.
      //System.out.println( scene.toString() );

      // Create a FrameBuffer to render our scene into.
      final int width  = 1024;
      final int height = 1024;
      final FrameBuffer fb = new FrameBuffer(width, height);

      Rasterize_Clip.doClipping = false; // turn off clipping

      for (int j = 0; j <= 100; ++j)
      {
         // Render again.
         fb.clearFB(Color.darkGray);
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_OfflineCubes_Frame%03d.ppm", j));

         // Update the scene by translating in
         // camera space each of the vertices.
         for (int i = 0; i < model1.vertexList.size(); ++i)
         {
            final Vertex v = model1.vertexList.get(i);
            model1.vertexList.set(i, new Vertex(v.x - 0.1,
                                                v.y - 0.03,
                                                v.z));
         }
         for (int i = 0; i < model2.vertexList.size(); ++i)
         {
            final Vertex v = model2.vertexList.get(i);
            model2.vertexList.set(i, new Vertex(v.x + 0.1,
                                                v.y,
                                                v.z));
         }
      }
   }
}
