/*

*/

import renderer.scene.*;
import renderer.pipeline.*;
import renderer.framebuffer.*;

import java.awt.Color;

/**
<pre>{@code
                  y       |
                  |       |
                  |       |
                  | v[1]  |
                1 +       |   /
                  |       |  /
                  |       | /
                  |       |/
                  |    -1 +--------------- image plane
                  |      /
                  |     /
                  |    /
                  |   /
                  |  /
                  | /
                  |/                v[0]
             v[2] +-----------------+------> x
                 /0                 1
                /
               /
              /
           1 + Camera
            /
           z
}</pre>
   Render a wireframe triangle. This is just about the
   simplest possible model. It is useful for debugging.
*/
public class OfflineTriangle
{
   public static void main(String[] args)
   {
      // Create the Scene object that we shall render.
      final Scene scene = new Scene();

      // Create a Model object to hold the geometry.
      final Model model = new Model("triangle");

      // Add the Model to the Scene.
      scene.addModel(model);

      // Create the geometry for the Model.
      final Vertex v0 = new Vertex(1.0, 0.0, 0.0);
      final Vertex v1 = new Vertex(0.0, 1.0, 0.0);
      final Vertex v2 = new Vertex(0.0, 0.0, 0.0);
    //final Vertex v2 = new Vertex(0.0, 0.0, 1.0);  // lands right on the camera
      model.addVertex(v0, v1, v2);

      // Add the geometry to the Model.
      model.addLineSegment(new LineSegment(0, 1),
                           new LineSegment(1, 2),
                           new LineSegment(2, 0));

      // Push the model away from where the camera is.
      for (int i = 0; i < model.vertexList.size(); ++i)
      {
         final Vertex v = model.vertexList.get(i);
         model.vertexList.set(i, new Vertex(v.x,
                                            v.y,
                                            v.z - 2));
      }

      // Log interesting information to standard output.
      //System.out.println( scene.toString() );

      // Create a FrameBuffer to render our scene into.
      final int width  = 1024;
      final int height = 1024;
      final FrameBuffer fb = new FrameBuffer(width, height);

      // Render our scene into the frame buffer once with debugging.
      fb.clearFB(Color.darkGray);
      Pipeline.debug  = true;
      Rasterize_Clip.debug = true;
      Pipeline.render(scene, fb);
      fb.dumpFB2File(String.format("PPM_OfflineTriangle_Frame%03d.ppm", 0));
      Pipeline.debug  = false;
      Rasterize_Clip.debug = false;
      Rasterize_Clip.doClipping = false; // turn off clipping

      for (int j = 1; j <= 60; ++j)
      {
         // Update the scene by translating in
         // camera space each of the 3 vertices.
         for (int i = 0; i < model.vertexList.size(); ++i)
         {
            final Vertex v = model.vertexList.get(i);
            model.vertexList.set(i, new Vertex(v.x + 0.2,
                                               v.y,
                                               v.z - 0.05));
         }

         // Render into the framebuffer again.
         fb.clearFB(Color.darkGray); // What happens if we don't clear the framebuffer?
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_OfflineTriangle_Frame%03d.ppm", j));
      }
   }
}
